#ifndef _H_MORPH_AST
#define _H_MORPH_AST

#include "loc.h"
#include "token.h"
#include <stddef.h>
typedef struct ast_op_s ast_op_t;

typedef struct const_s {
    const char* name;
    token_t val;
} const_t;

typedef struct memory_s {
    const char* name;
    size_t size;
} memory_t;

typedef struct ast_if_stat_s {
    ast_op_t* condition;
    ast_op_t* body;
    bool is_elseif;

    union {
        struct ast_if_stat_s* elseif;
        ast_op_t* else_body;
    };
} ast_if_stat_t;

typedef struct ast_while_stat_s {
    ast_op_t* condition;
    ast_op_t* body;
} ast_while_stat_t;

typedef enum ast_op_type_e { AOT_OP, AOT_IF, AOT_WHILE, AOT_USE_CONST, AOT_USE_MEMORY, AOT_CALL_FUNC } ast_op_type_t;

typedef struct ast_op_s {
    ast_op_type_t type;
    loc_t loc;

    union {
        token_t op;
        ast_if_stat_t if_stat;
        ast_while_stat_t while_stat;
        size_t id;
        char* func_name;
    };
} ast_op_t;

typedef struct function_s {
    loc_t loc;
    const char* name;
    const char** args;
    const char** return_args;
    ast_op_t* body;
} function_t;

typedef struct program_s {
    const_t* const_vars;
    function_t* funcs;
    memory_t* memories;
} program_t;

#endif // !_H_MORPH_AST
